/*
	Sokonux - 1.0.3
  Copyright (C) 2007  Daniel Ponsoda Montiel

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "Caja.h"
#include "Sistema.h"

/**
 * Constructor de la clase
 * Solo inicializa a cero los atributos casX, casY y moviendo
 */
Caja::Caja()
{
  // Inicializar los atributos por defecto
  casX=0;
  casY=0;
  moviendo=0;
};

/**
 * Destructor de la clase caja
 */
Caja::~Caja(){

}

/**
 * Solicita el movimiento de una caja hacia una direccion indicada.
 * No tiene por que moverse. Ello dependera de las reglas del movimiento
 * de la caja que se determinan en este metodo.
 * @param dir Direccia la que se quiere mover: IZQ = izquierda; DER = derecha;
 *        ARR = arriba; ABA = abajo
 */
int Caja::mover(int dir){
  int nuevaCas0=0, nuevaCas1=0;

  // Si ya se esta moviendo, no hacer nada.
  if (moviendo) return 0;

  moviendo=dir;

  // Comprobar que es lo que hay en la direccion en la que se va a mover
  switch (dir){
    case IZQ: // Si se ha llegado al limite izquierdo, no mover
              if (casY<=0) {moviendo=0; return 0;}
              // Obtener lo que hay en el suelo en esa direccion
              nuevaCas0 = Mapa::escena[0][casY-1][casX].tipo;
              // Obtener lo que hay delante en esa direccion
              nuevaCas1 = Mapa::escena[1][casY-1][casX].tipo;
              break;

    case DER: // Si se ha llegado al limite derecho, no mover
              if (casY>=MAX_LARGO-1) {moviendo=0; return 0;}
              // Obtener lo que hay en el suelo en esa direccion
              nuevaCas0 = Mapa::escena[0][casY+1][casX].tipo;
              // Obtener lo que hay delante en esa direccion
              nuevaCas1 = Mapa::escena[1][casY+1][casX].tipo;
              break;

    case ARR: // Si se ha llegado al limite hacia arriba, no mover
              if (casX<=0) {moviendo=0; return 0;}
              // Obtener lo que hay en el suelo en esa direccion
              nuevaCas0 = Mapa::escena[0][casY][casX-1].tipo;
              // Obtener lo que hay delante en esa direccion
              nuevaCas1 = Mapa::escena[1][casY][casX-1].tipo;
              break;

    case ABA: // Si se ha llegado al limite hacia abajo, no mover
              if (casX>=MAX_ANCHO-1) {moviendo=0; return 0;}
              // Obtener lo que hay en el suelo en esa direccion
              nuevaCas0 = Mapa::escena[0][casY][casX+1].tipo;
              // Obtener lo que hay delante en esa direccion
              nuevaCas1 = Mapa::escena[1][casY][casX+1].tipo;
              break;
  }

  // Si no hay nada en la nueva casilla, no mover
  if (nuevaCas0==IMG_VACIO) { moviendo = 0; return 0; }

  // Si hay una pared, no mover
  if (nuevaCas1==IMG_PARED) { moviendo = 0; return 0; }

  // Si es una caja que mueve a otra caja, no mover
  if ((nuevaCas1==IMG_CAJA) && (Mapa::escena[1][casY][casX].tipo==IMG_CAJA)){
    moviendo = 0;
    return 0;
  }

  return 1;
}


/**
 * Este es el metodo que se encarga de ejecutar el movimiento de los objetos
 * cuando se estan moviendo (atributo moviendo distinto de 0).
 * La llamada a este metodo se repetira en el bucle principal del juego,
 * mientras que la llamada al metodo mover, solo se realiza una vez cuando
 * el usuario pulsa la tecla correspondiente.
 */
int Caja::controlarMovimiento()
{
  int resultado=0;
  // Creamos punteros temporales a las variables de la casilla
  // Con el fin de abreviar el nombre, y que el codigo sea mas legible
  int* tmpX     = &Mapa::escena[1][casY][casX].desX;
  int* tmpY     = &Mapa::escena[1][casY][casX].desY;
  int* tmpTipo  = &Mapa::escena[1][casY][casX].tipo;

  //
  // Si se esta moviendo, mueve el objeto
  // una casilla hacia la direccion indicada
  //
  switch (moviendo){
    // Mover a la izquierda
    //
    case IZQ:
      // Alterar las posiciones de desvio
      *tmpX-=2;
      *tmpY-=1;
      // Si ya se ha desplazado una casilla entera
      if ((*tmpX<=-30) || (*tmpY<=-15)){
        // Poner las posiciones de desvio a 0 ...
        *tmpX=0;
        *tmpY=0;
        // ... y avanzar una casilla
        casY--;
        // ... indicando el contenido en la nueva posicion
        Mapa::escena[1][casY][casX].tipo = *tmpTipo;
        // ... y vaciando la posicion anterior
        *tmpTipo=0;
        // Finalmente, detener el movimiento
        moviendo=0;
        resultado=1;
      }
    break;

    // Mover a la derecha
    //
    case DER:
      // Alterar las posiciones de desvio
      *tmpX+=2;
      *tmpY+=1;
      // Si ya se ha desplazado una casilla entera
      if ((*tmpX>=30) || (*tmpY>=15)){
        // Poner las posiciones de desvio a 0 ...
        *tmpX=0;
        *tmpY=0;
        // ... y avanzar una casilla
        casY++;
        // ... indicando el contenido en la nueva posicion
        Mapa::escena[1][casY][casX].tipo = *tmpTipo;
        // ... y vaciando la posicion anterior
        *tmpTipo=0;
        // Finalmente, detener el movimiento
        moviendo=0;
        resultado=1;
      }
    break;

    // Mover hacia arriba
    //
    case ARR:
      // Alterar las posiciones de desvio
      *tmpX+=2;
      *tmpY-=1;
      // Si ya se ha desplazado una casilla entera
      if ((*tmpX>=30) || (*tmpY<=-15)){
        // Poner las posiciones de desvio a 0 ...
        *tmpX=0;
        *tmpY=0;
        // ... y avanzar una casilla
        casX--;
        // ... indicando el contenido en la nueva posicion
        Mapa::escena[1][casY][casX].tipo = *tmpTipo;
        // ... y vaciando la posicion anterior
        *tmpTipo=0;
        // Finalmente, detener el movimiento
        moviendo=0;
        resultado=1;
      }
    break;

    // Mover hacia abajo
    case ABA:
      // Alterar las posiciones de desvio
      *tmpX-=2;
      *tmpY+=1;
      // Si ya se ha desplazado una casilla entera
      if ((*tmpX<=-30) || (*tmpY>=15)){
        // Poner las posiciones de desvio a 0 ...
        *tmpX=0;
        *tmpY=0;
        // ... y avanzar una casilla
        casX++;
        // ... indicando el contenido en la nueva posicion
        Mapa::escena[1][casY][casX].tipo = *tmpTipo;
        // ... y vaciando la posicion anterior
        *tmpTipo=0;
        // Finalmente, detener el movimiento
        moviendo=0;
        resultado=1;
      }
  }
  return resultado;
}

