/*
	Sokonux - 1.0.3
  Copyright (C) 2007  Daniel Ponsoda Montiel

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "CuadroTexto.h"
#include "Sistema.h"
#include <string.h>

/**
 * Constructor de la clase CuadroTexto
 * @param cuadroPadre Indica cual es su cuadro padre (en general a que ventana
 *                    pertenece). Si esta a 0, el padre es la pantalla
 * @param x           Posicion horizontal del boton relativa al padre
 * @param y           Posicion vertical del boton relativa al padre
 * @param ancho       Anchura del cuadro de texto
 * @param alto        Altura del cuadro de texto
 * @param maxLetras   Maximo numero de caracteres que se pueden escribir
 */
CuadroTexto::CuadroTexto (CUADRO** cuadroPadre, int x, int y,
                          int ancho, int alto, int maxLetras)
           : Widget (cuadroPadre,x,y,ancho,alto)
{
  // Inicializar los atributos por defecto
  colorFondo    = DF_COLORFONDOCT;
  colorBorde    = DF_COLORBORDECT;
  colorTexto    = DF_COLORTEXTOCT;
  flags         = DF_FLAGSCT;
  graficos      = NULL;
  longitud      = maxLetras;
  posicionActual= 0;
  bufferCero    = 0;
  foco          = this;
  cambiado      = 0;

  // Reservar espacio para la cadena de texto
  textoEscrito  = new char[maxLetras+1];
  textoEscrito[0]=0;

}

/**
 * Destructor de la clase CuadroTexto
 */
CuadroTexto::~CuadroTexto()
{
  // Libera el texto escrito, si habia.
  if (textoEscrito) delete textoEscrito;
}

/**
 * Cambia el contenido del texto dentro del cuadro
 * @param cadena Nueva cadena de texto.
 */
void CuadroTexto::cambiarTexto(char* cadena)
{
  if (textoEscrito) delete textoEscrito;
  textoEscrito = new char[longitud+1];
  cadena[longitud]='\0';
  strcpy(textoEscrito, cadena);
  cambiado=0;
  posicionActual = strlen(textoEscrito);
}

/**
 * Dibuja el cuadro de texto
 */
void CuadroTexto::dibujar()
{
  unsigned char letra;

  if (detectarRaton()==MSG_RATON_CLICK) foco=this;

  // Dibujar el cuadro
  Widget::dibujar();

  // Si no tiene el foco, comprobar lo que escribe el usuario
  if (foco==this){
    letra=Sistema::comprobarTeclado();

    // Si el buffer del teclado no esta a cero, vaciar el buffer
    if (!bufferCero){
      do{
        letra=Sistema::comprobarTeclado();
      }while(letra!=0);
      bufferCero=1;
    }

    // Escribir una letra
    if ((letra>=32) && (letra<=127) && (posicionActual<longitud)){
      cambiado=1;
      textoEscrito[posicionActual]=letra;
      textoEscrito[posicionActual+1]='\0';
      posicionActual++;
    }

    // Borrar una letra
    if ((letra==TECLA_DEL) && (posicionActual>0)){
      cambiado=1;
      textoEscrito[posicionActual-1]='\0';
      posicionActual--;
    }
  }

  // Escribir el texto
  if (textoEscrito[0])
    Sistema::escribirTexto(textoEscrito,xAbs(cuadro)+5,
                           yAbs(cuadro)+2,0,ARIAL12);
}
