/*
	Sokonux - 1.0.3
  Copyright (C) 2007  Daniel Ponsoda Montiel

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "Editor.h"

/**
 * Constructor de la clase Editor
 */
Editor::Editor()
{
#ifdef DEBUG
  puts("Inicializando Editor");
#endif

  nivelActual       = 0;
  herramientaActual = HER_SUELO;

  // Crear el mapa
  mapa=new Mapa();
  mapa->jugando=0;

  // Inicializar los flags para menu archivo
  nuevoMapa = 0;
  cambios   = 0;

  // Crear las ventanas
  crearVentanas();
}

/**
 * Destructor de la clase Editor
 */
Editor::~Editor()
{

}

/**
 * Sale del editor (tras confirmacion)
 * @return Devuelve la respuesta del usuario
 */
int Editor::terminar()
{
  // Pedir confirmacion
  if (mostrarMensaje(TXT_SOKONUX,TXT_SALIREDITOR,0)==MSG_ACEPTAR)
    m_salir=-1;
  else
    m_salir=0;

  return m_salir;
}

/**
 * Actualiza la pantalla del editor
 */
void Editor::actualizarPantalla()
{
  char cadena[200];
  // Mostrar numero de nivel actual
  sprintf(cadena,"Editando nivel: %d de %d",nivelActual+1,mapa->nNiveles);
  Sistema::escribirTexto(cadena,150,100,0xFFFFFF,ARIAL12);

  // Dibujar el mapa
  if (mapa) mapa->    dibujar();

  // Dibujar ventanas
  cajaHerramientas->  dibujar();
  menuArchivo->       dibujar();
  menuPropiedades->   dibujar();

}

/**
 * Comprobacion de entrada (principalmente raton) para el editor
 */
void Editor::comprobarEntrada(){
  // El metodo padre comienza a comprobar la entrada
  Programa::comprobarEntrada();

  // Si esta sobre una casilla, poner la casilla
  // correspondiente a la herramienta
  comprobarCasillas();

  // Si esta sobre una herramienta, seleccionarla
  comprobarHerramientas();

  // Si esta sobre una opcion del menu, seleccionarla
  comprobarArchivo();

}

/**
 * Controla las opciones de la caja de herramientas
 */
void Editor::comprobarHerramientas()
{
  int i;
  int arrayHerramientas[6] =
      {HER_VACIO,HER_SUELO,HER_PARED,HER_OBJETIVO,HER_CAJA,HER_PERSONAJE};

  for (i=0;i<6;i++)
  {
    if (cajaHerramientas->boton[i]->detectarRaton()==MSG_RATON_CLICK)
      herramientaActual=arrayHerramientas[i];
  }
}

/**
 * Controla las opciones del menu archivo
 */
void Editor::comprobarArchivo()
{
  // Boton nuevo
  if (menuArchivo->boton[0]->detectarRaton()==MSG_RATON_CLICK){
    // Si el mapa no es nuevo
    if (!nuevoMapa)
    {
      // Preguntar si se quiere guardar
      if (guardarCambios()==MSG_CANCELAR) return;

      // Crear un nuevo nivel
      nivelActual=mapa->nNiveles;
      mapa->crearMapaVacio();
      nuevoMapa=1;
      menuPropiedades->cuadroTexto[0]->cambiarTexto(mapa->puntos);
      menuPropiedades->cuadroTexto[1]->cambiarTexto(mapa->password);
      if (menuPropiedades->cuadroTexto[0]->textoEscrito[0]==0);
    }
  }

  // Boton guardar
  if (menuArchivo->boton[1]->detectarRaton()==MSG_RATON_CLICK){
    // Preguntar si se quiere guardar
    if (guardarCambios()==MSG_CANCELAR) return;
  }

  // Boton cargar
  if (menuArchivo->boton[2]->detectarRaton()==MSG_RATON_CLICK)
  {
    // Preguntar si se quiere guardar
    if (guardarCambios()==MSG_CANCELAR) return;
    cargarNivel();
  }

  // Boton salir
  if (menuArchivo->boton[3]->detectarRaton()==MSG_RATON_CLICK){
    // Preguntar si se quiere guardar
    int res=guardarCambios();
    if (res==MSG_CANCELAR) return;

    // Si no se mostro el mensaje (No habian cambios)
    if (res==0)
      // Confirmar para salir
      m_salir=-1;
    else
      // Salir sin confirmar de nuevo
      m_salir=-2;
  }
}

/**
 * Controla la edicion de las casillas
 */
void Editor::comprobarCasillas()
{
  int i,j;
  int x=mapa->rX;
  int y=mapa->rY;
  int z=mapa->rZ;

  // Si no se ha pulsado el raton, no comprobar nada
  if (!Sistema::comprobarBotonesRaton()) return;

  // Si el cursor no esta sobre el tablero, no comprobar nada
  if ((x<0) || (y<0)) return;

  // El mapa va a ser modificado
  nuevoMapa=0;
  cambios=1;

  // Comprobar las herramientas
  switch (herramientaActual)
  {
    // Herramienta borrar
    case HER_VACIO:
      for (i=z;i<MAX_ALTO;i++){
        mapa->escena[i][y][x].tipo = HER_VACIO;
      }
    break;

    // Herramientas suelo y objetivo
    case HER_SUELO:
    case HER_OBJETIVO:
      mapa->escena[0][y][x].tipo = herramientaActual;
      mapa->escena[1][y][x].tipo = HER_VACIO;
    break;

    // Herramienta personaje
    case HER_PERSONAJE:
      for (j=0;j<MAX_LARGO;j++)
        for (i=0;i<MAX_ANCHO;i++)
          if (mapa->escena[1][j][i].tipo == HER_PERSONAJE)
            mapa->escena[1][j][i].tipo = 0;

      mapa->escena[1][y][x].tipo = HER_PERSONAJE;

      // Poner suelo si no hay nada
      if (!mapa->escena[0][y][x].tipo)
        mapa->escena[0][y][x].tipo = HER_SUELO;

    break;

    // Resto de herramientas
    default:
      mapa->escena[1][y][x].tipo = herramientaActual;

      // Poner suelo si no hay nada
      if (!mapa->escena[0][y][x].tipo)
        mapa->escena[0][y][x].tipo = HER_SUELO;
  }
}

/**
 * Carga un nivel
 * Si se indica un numero por parametro, carga el nivel correspondiente.
 * Si no, se solicita al usuario el numero de nivel que quiere cargar.
 * Luego carga el nivel solicitado.
 * @param numero Numero de nivel a cargar
 */
void Editor::cargarNivel(int numero)
{
  puts("Cargando el nivel");
  char *nivel;
  int res=0;

  // Si el numero de nivel se ha pasado por parametro
  if (numero>=0){
    res=mapa->cargar(numero);
  }
  // Si no, pedirlo al usuario
  else{
    Texto* tmpTexto;
    Sistema::textof(&tmpTexto, TXT_INTRNUMERO, mapa->nNiveles);
    if (pedirInformacion(TXT_CARGAR,tmpTexto,&nivel,0)!=MSG_CANCELAR)
    {
      res=mapa->cargar(atoi(nivel)-1);
    }
    // Si el usuario cancela, volver
    else return;
  }

  // Si no se ha cargado correctamente
  if (res<0){
    if (numero==0)
      // No hay niveles
      mostrarMensaje(TXT_EDITOR,TXT_NONIVELES, MSG_ACEPTAR);
    else
      // No se ha cargado el nivel solicitado
      mostrarMensaje(TXT_ERROR,TXT_NOCARGADO, MSG_ACEPTAR);
  }
  // Si se ha cargado correctamente
  else {
    // Cambiar el texto del menu propiedades con los nuevos datos
    menuPropiedades->cuadroTexto[0]->cambiarTexto(mapa->puntos);
    menuPropiedades->cuadroTexto[1]->cambiarTexto(mapa->password);
    nuevoMapa=0;
    cambios=0;
    nivelActual=res;
  }
  puts("Nivel cargado");
}


/**
 * Guarda los cambios realizados, previa confirmacion del usuario
 * mostrando mensajes de error si hay algun error de validacion en el mapa
 * @return 0 = No habian camibos que guardar
 *         Distinto de 0 = La respuesta del usuario
 */
DWORD Editor::guardarCambios()
{
  DWORD mensaje;
  // Si no han habido cambios, no hacer nada
  if ((!cambios) && (!menuPropiedades->cuadroTexto[0]->cambiado)
                 && (!menuPropiedades->cuadroTexto[1]->cambiado))
  return 0;

  strcpy(mapa->puntos,  menuPropiedades->cuadroTexto[0]->textoEscrito);
  strcpy(mapa->password,menuPropiedades->cuadroTexto[1]->textoEscrito);
  switch (mapa->validarMapa()){
    case 0:
      mensaje = mostrarMensaje(TXT_GUARDAR, TXT_CONFGUARDAR,MSG_SINOCAN);
      if (mensaje==MSG_ACEPTAR)
      {
        if (mapa->guardar(nivelActual)!=0)
          mostrarMensaje(TXT_ERROR,TXT_NOGUARDADO, MSG_ACEPTAR);
        else{
          cambios=0;
          menuPropiedades->cuadroTexto[0]->cambiado=0;
          menuPropiedades->cuadroTexto[1]->cambiado=0;
          nuevoMapa=0;
          mostrarMensaje(TXT_GUARDAR, TXT_GUARDADO, MSG_ACEPTAR);
        }
      }
      return mensaje;
      break;

    case 1: mostrarMensaje(TXT_ERROR, TXT_NOPERSONAJE,  MSG_ACEPTAR); break;
    case 2: mostrarMensaje(TXT_ERROR, TXT_NOCAJAS,      MSG_ACEPTAR); break;
    case 3: mostrarMensaje(TXT_ERROR, TXT_CAJANOOBJET,  MSG_ACEPTAR); break;
    case 4: mostrarMensaje(TXT_ERROR, TXT_CAJAOBJETIVO, MSG_ACEPTAR); break;
    case 5: mostrarMensaje(TXT_ERROR, TXT_NOPASSWORDED, MSG_ACEPTAR); break;
  }

  return 0;
}

/**
 * Prepara todas las ventanas del editor al construir la clase
 */
void Editor::crearVentanas()
{
  // Crear el menu "Herramientas"
  cajaHerramientas = new Ventana(0,TXT_HERRAMIENTAS,
                                 20,20,100,400,WID_ARRASTRABLE);

  cajaHerramientas->agregarBoton("img/bloque00.bmp", 20, 20);
  cajaHerramientas->agregarBoton("img/bloque02.bmp", 20, 70);
  cajaHerramientas->agregarBoton("img/bloque03.bmp", 20, 120);
  cajaHerramientas->agregarBoton("img/bloque04.bmp", 20, 170);
  cajaHerramientas->agregarBoton("img/bloque05.bmp", 20, 220);
  cajaHerramientas->agregarBoton("img/personaje.bmp", 20, 300);

  // Crear el menu "Archivo"
  menuArchivo = new Ventana(0,TXT_ARCHIVO,900,20,100,270,WID_ARRASTRABLE);
  menuArchivo->colorFondo=0x555555;

  menuArchivo->agregarBoton(TXT_NUEVO,20,30,60,30);
  menuArchivo->agregarBoton("img/icono01.bmp",20,70);
  menuArchivo->agregarBoton("img/icono02.bmp",20,130);
  menuArchivo->agregarBoton("img/icono03.bmp",20,190);

  // Crear el menu "Propiedades del nivel"
  menuPropiedades = new Ventana(0,TXT_PROPIEDADES,20,500,400,100,
                                WID_ARRASTRABLE);

  menuPropiedades->agregarTexto(TXT_PUNTOSEDIT,10,10);
  menuPropiedades->agregarTexto(TXT_PASSWORDEDIT,10,40);
  menuPropiedades->agregarCuadroTexto(150,8,100,25,10);
  menuPropiedades->agregarCuadroTexto(150,38,100,25,10);

}

/**
 * Bucle principal del editor
 */
int Editor::bucle()
{
  puts("Entrando en el bucle del editor");
  nivelActual=0;
  nuevoMapa=0;

  cargarNivel(nivelActual);

  Programa::bucle();

  return 0;
}

