/*
	Sokonux - 1.0.3
  Copyright (C) 2007  Daniel Ponsoda Montiel

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef Editor_h
#define Editor_h

#include "Programa.h"
#include "Mapa.h"

// Constantes de las herramientas
#define HER_VACIO         IMG_VACIO
#define HER_MARCAR        IMG_MARCAR
#define HER_SUELO         IMG_SUELO
#define HER_PARED         IMG_PARED
#define HER_OBJETIVO      IMG_OBJETIVO
#define HER_CAJA          IMG_CAJA
#define HER_PERSONAJE     IMG_PERSONAJE
#define HER_BORRAR        7
#define HER_SELECCIONAR   8
#define MAX_HERRAMIENTAS  9

/**
 * Clase Editor heredada de Programa
 * Es el subprograma editor de niveles
 */
class Editor : public Programa {

 public:
    // Constructor y destructor
    Editor();
    virtual ~Editor();

    // Comprueba la entrada por raton y teclado
    void comprobarEntrada();

    // Bucle principal del editor
    int bucle();

 public:
    // Herramienta seleccionada actualmente
    int herramientaActual;

    // Nivel que se esta editando
    int nivelActual;

 private:
    // Crea e inicializa las ventanas del editor
    void  crearVentanas();

    // Comprueba si se esta tocando una casilla
    void  comprobarCasillas();

    // Comprueba la caja de herramientas
    void  comprobarHerramientas();

    // Comprueba el menu archivo
    void  comprobarArchivo();

    // Guarda los cambios en el nivel
    DWORD guardarCambios();

    // Carga un nivel
    void  cargarNivel(int numero=-1);

    // Actualiza la pantalla del editor
    void actualizarPantalla();

    // Sale del editor
    int   terminar();

 private:
    // Ventana de la caja de herramientas
    Ventana* cajaHerramientas;

    // Ventana del menu archivo
    Ventana* menuArchivo;

    // Ventana del menu propiedades
    Ventana* menuPropiedades;

    // Mapa actual
    Mapa* mapa;

    // Flags para el control de los cambios
    bool nuevoMapa;
    bool cambios;
};

#endif
