/*
	Sokonux - 1.0.3
  Copyright (C) 2007  Daniel Ponsoda Montiel

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "Juego.h"

/**
 * Constructor de la clase Juego
 */
Juego::Juego(){

#ifdef DEBUG
  puts("Inicializando Juego");
#endif

  // El nivel actual es el 0
  nivelActual=0;

  // Crear el mapa
  mapa = new Mapa();
  records = new Records();
}

/**
 * Destructor de la clase Juego
 */
Juego::~Juego(){

}

/**
 * Comprueba la entrada de teclado y raton durante la partida
 */
void Juego::comprobarEntrada(){

  // El metodo padre comienza la comprobacion de entrada
  Programa::comprobarEntrada();

  if (Sistema::comprobarTeclado(TECLA_IZQ)) personaje->mover(IZQ);
  if (Sistema::comprobarTeclado(TECLA_DER)) personaje->mover(DER);
  if (Sistema::comprobarTeclado(TECLA_ARR)) personaje->mover(ARR);
  if (Sistema::comprobarTeclado(TECLA_ABA)) personaje->mover(ABA);

  // Pasar de nivel (truco) pulsando s y l a la vez
  if ((Sistema::comprobarTeclado(SDLK_s))
      && (Sistema::comprobarTeclado(SDLK_l,500)))
  {
    pasarNivel();
    //puntos=0;
    return;
  }

  // Comprobar si se solicita reiniciar el nivel
  if (((TXT_REINICIAR->detectarRaton()==MSG_RATON_CLICK) ||
      (Sistema::comprobarTeclado(SDLK_r))) && (pasos>0)){
    reintentos++;
    prepararNivel();
  }

  // Mover al personaje con el raton
  if (Sistema::comprobarBotonesRaton())
    personaje->moverR(mapa->rX, mapa->rY);
}

/**
 * Comprueba y controla el estado de la partida
 */
void Juego::comprobarEstado(){

  // Controlar el movimiento de todas las cajas
  for (int i=0; i<nCajas; i++) caja[i]->controlarMovimiento();

  // Controlar el movimiento del personaje
  if (personaje->controlarMovimiento()!=0)
    pasos++;

  // Comprobar los objetivos
  if (mapa->comprobarObjetivos() == nCajas)
  {
    pasarNivel();
    return;
  }

  // Controlar el tiempo
  if ((Sistema::reloj()-inicio)>PASOS_POR_SEGUNDO){
    inicio = Sistema::reloj();
    tiempo++;
  }

  // El metodo padre termina de comprobar el estado
  Programa::comprobarEstado();
}

/**
 * Actualiza la pantalla de juego
 */
void Juego::actualizarPantalla()
{
  TXT_NIVELACTUAL->dibujarFormateado(0,nivelActual+1);
  TXT_PUNTOSNIVEL->dibujarFormateado(0,mapa->puntos);
  TXT_TIEMPOJUEGO->dibujarFormateado(0,tiempo);
  TXT_PASOSJUEGO->dibujarFormateado(0,pasos);
  TXT_PUNTOSJUEGO->dibujarFormateado(0,puntos);
  TXT_REINIJUEGO->dibujarFormateado(0,reintentos);

  TXT_PASWDJUEGO->dibujarFormateado(0,mapa->password);
  TXT_REINICIAR->dibujar();

  // Dibujar el mapa
  if (mapa) mapa->dibujar();

}

/**
 * Sale del juego (tras confirmacion)
 * @return Devuelve la respuesta del usuario
 */
int Juego::terminar()
{
  if (mostrarMensaje(TXT_SOKONUX,TXT_ABANDONAR,0) == MSG_ACEPTAR){
    //records->comprobar(puntos);
    m_salir=-1;
  }
  else
    m_salir=0;

  return m_salir;
}

/**
 * Pasa de nivel y contabiliza los puntos conseguidos
 */
void Juego::pasarNivel()
{
  int puntActual = atoi(mapa->puntos) - tiempo - (2*pasos);
  if (puntActual<100) puntActual=100;
  puntos+=puntActual;

  nivelActual++;
  m_salir=nivelActual;
}

/**
 * Prepara el nivel actual
 */
void Juego::prepararNivel()
{
  int i,j;
  int cont=0;
  // Cargar el mapa del nivel actual
  if (mapa->cargar(nivelActual)<0){
    bucleSinNiveles();
    return;
  }

  // Resetear los parametros del nivel
  inicio = Sistema::reloj();
  tiempo = 0;//atoi(mapa->tiempo);
  pasos  = 0;
  nCajas = mapa->contarCajas();

  // Preparar las cajas
  if (caja) delete caja;
  caja = new Caja*[nCajas];

  // Buscar la posicion de las cajas
  for (j=0;j<MAX_LARGO;j++)
    for (i=0;i<MAX_ANCHO;i++)
      if (mapa->escena[1][j][i].tipo == IMG_CAJA){

        caja[cont] = new Caja();
        caja[cont]->casX = i;
        caja[cont]->casY = j;
        cont++;
      }

  // Preparar el personaje
  personaje = new Personaje(caja, nCajas);

  // Buscar la posicion del personaje en el mapa
  for (j=0;j<MAX_LARGO;j++)
    for (i=0;i<MAX_ANCHO;i++)
      if (mapa->escena[1][j][i].tipo == IMG_PERSONAJE){
        personaje->casX=i;
        personaje->casY=j;
      }
}

/**
 * Comprueba el password cuando se solicita continuar una partida
 * @return -1 = El usuario ha cancelado.
 *          0 = El password es correcto y se ha cargado un nivel.
 */
int Juego::comprobarPassword()
{
  char* password;
  int botonPulsado;
  Texto* mensaje=TXT_PASSWORD;

  do{
    botonPulsado = pedirInformacion(TXT_CONTINUAR, mensaje, &password, 0);

    if (botonPulsado==MSG_CANCELAR) {
      if (terminar()) return -1;
    }
    mensaje=TXT_NOPASSWORD;

  // Asignar a nivelActual el valor devuelto al cargar el mapa
  // Y comparar si es menor que 0
  }while( (nivelActual=mapa->cargar(password)) < 0);

  return 0;
}

/**
 * Este bucle no muestra la pantalla de juego, sino un mensaje de enhorabuena
 * cuando se han acabado los niveles o un mensaje de error si no se pudo cargar
 * el fichero de niveles.
 */
void Juego::bucleSinNiveles()
{
  m_salir=0;
  Sistema::pausa(m_pausa);
  Sistema::vaciarBufferTeclado();

  do{
    Sistema::volcarPantalla();
    // Si no hay niveles
    if (nivelActual<=0) TXT_NONIVELJUEGO->dibujar();

    // Si el jugador ha pasado todos los niveles
    else                TXT_ENHORABUENA->dibujar();

    if ((Sistema::comprobarBotonesRaton()) || (Sistema::comprobarTeclado())){
      m_salir=-2;
      nivelActual=-1;
    }

  }while(!m_salir);

}

/**
 * Bucle principal del juego
 * @param continuar 0 = Si se empieza una nueva partida
 *                  1 = Si se desea continuar una anterior
 */
int Juego::bucle(int continuar)
{
  // Inicializar los parametros del nivel
  nivelActual = 0;
  reintentos  = 0;
  puntos      = 0;

  // Si se solicita continuar, pedir password
  if (continuar)
    if (comprobarPassword()!=0) return -1;

  // Bucle principal del juego
  do{

    prepararNivel();

    if (nivelActual>=0)
      nivelActual=Programa::bucle();

  // nivelActual ser -1 cuando el usuario solicite terminar la partida
  }while(nivelActual>=0);

  // Al finalizar, se comprueba si se ha superado un record
  records->comprobar(puntos);

  return 0;
}
