/*
	Sokonux - 1.0.3
  Copyright (C) 2007  Daniel Ponsoda Montiel

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef Mapa_h
#define Mapa_h

#include "Casilla.h"

// Constantes de las imagenes del mapa
#define IMG_VACIO     0
#define IMG_MARCAR    1
#define IMG_SUELO     2
#define IMG_PARED     3
#define IMG_OBJETIVO  4
#define IMG_CAJA      5
#define IMG_PERSONAJE 6
#define IMG_CAJA2     7
#define MAX_IMAGENES  8

// Constantes de las dimensiones
#define MAX_ALTO      2
#define MAX_ANCHO    12
#define MAX_LARGO    12

// Raton detectado en una casilla
#define RATON_DETECTADO 0x010000

// Estructura empleada para controlar el mapa
typedef struct ESCENA {
  int tipo;   // Tipo de casilla
  int desX;   // Desvio horizontal
  int desY;   // Desvio vertical
};

// Estructura empleada para ordenar y dibujar el mapa
typedef struct ESCORDENADA {
  int tipo;   // Tipo de casilla
  int desX;   // Posicion horizontal
  int desY;   // Posicion vertical
  int raton;  // Flag que indica si esta el raton
  int ver;    // Flag que indica si es visible
};

/**
 * Clase Mapa.
 * Se encarga de crear, cargar, guardar, mostrar, consultar y validar el mapa.
 */
class Mapa {

  public:
    // Constructor y destructor de la clase
    Mapa                        ();
    virtual ~Mapa               ();

  public:
    // Dibuja el mapa en pantalla
    void    dibujar             ();
    // Crea un mapa vacio. Se usa para inicializar y para crear un nuevo mapa
    void    crearMapaVacio      ();
    // Carga un mapa desde disco indicando el numero de nivel
    int     cargar              (int nivel);
    // Carga un mapa desde disco indicando su password
    int     cargar              (char* pass);
    // Guarda el mapa actual en disco indicando el numero de nivel
    int     guardar             (int nivel);
    // Devuelve cuantas cajas hay en los objetivos (util para el juego)
    int     comprobarObjetivos  ();
    // Comprueba que el mapa sea correcto (usado en el editor antes de guardar)
    int     validarMapa         ();
    // Detecta en que casilla esta apuntando el raton
    void    detectarRaton       ();
    // Ordena todos los objetos por profundidad (usado antes de dibujar el mapa)
    void    ordenarProfundidad  ();
    // Cuenta el numero de cajas que hay en el mapa
    int     contarCajas         ();

  public:
    // Numero total de niveles
    int nNiveles;

  public:
    // Imagenes empleadas para dibujar el mapa
    Casilla *casilla[MAX_IMAGENES];

    // Datos del nivel
    char          puntos  [11];
    char          pasos   [11];
    char          password[11];
    static ESCENA escena  [MAX_ALTO][MAX_LARGO][MAX_ANCHO];

    // Lista de casillas ordenada por profundidad para facilitar su dibujado
    ESCORDENADA escOrdenada[MAX_ALTO*MAX_LARGO*MAX_ANCHO];

    // Indica si se esta jugado (Util para el modo en que seala el raton)
    bool jugando;

    // Variables para indicar en que casilla del mapa esta el raton
    int rX, rY, rZ;
};
#endif
