/*
	Sokonux - 1.0.3
  Copyright (C) 2007  Daniel Ponsoda Montiel

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "Personaje.h"
#include "Sistema.h"

/**
 * Constructor de la clase Personaje
 * @param caja Puntero a la instancia de la clase Caja que contiene las
 *             cajas que hay actualmente en el mapa
 * @param nCajas Numero total de cajas actuales.
 */
Personaje::Personaje(Caja** caja, int nCajas){
  this->caja=caja;
  this->nCajas=nCajas;
  destinoX=-1;
  destinoY=-1;

}

/**
 * Destructor de la clase Personaje
 */
Personaje::~Personaje(){

}


/**
 * Solicita el movimiento del personaje hacia una direccion indicada.
 * No tiene por que moverse. Ello dependera de las reglas del movimiento
 * del personaje que se determinan en este metodo.
 * @param dir Direccia la que se quiere mover: IZQ = izquierda; DER = derecha;
 *        ARR = arriba; ABA = abajo
 * @return Devuelve 0 si no se mueve, o 1 si se mueve.
 */
int Personaje::mover(int dir){
  Caja* tmpCaja=0; 

  if (moviendo) return 0;

  // El metodo padre comienza a revisar las reglas de movimiento generales
  if (!Caja::mover(dir)) return 0;

  // Y ahora se continuan aplicando las reglas especificas del personaje ...

  // Comprobar si hay una caja en la direccion a donde se mueve
  switch (dir){
    case IZQ: tmpCaja = detectarCaja(casX,casY-1); break;
    case DER: tmpCaja = detectarCaja(casX,casY+1); break;
    case ARR: tmpCaja = detectarCaja(casX-1,casY); break;
    case ABA: tmpCaja = detectarCaja(casX+1,casY); break;
  }

  // Si hay una caja, tratar de moverla
  if (tmpCaja){
    // Si no se puede empujar la caja, no mover el personaje
    if (!tmpCaja->mover(dir)){
      moviendo=0;
      return 0;
    }
  }

  return 1;
}

/**
 * Solicita el movimiento del personaje hacia una posicion indicada.
 * @param x Posicion horizontal en casillas isometricas.
 * @param y Posicion vertical en casillas isometricas.
 * @return Devuelve 0 si no se mueve, o 1 si se mueve.
 */
int Personaje::moverR(int x, int y)
{
  if ((x<0) || (y<0) || (x>MAX_ANCHO-1) || (y>MAX_LARGO-1)) return 0;

  if ((casX==x) && (casY==y)) {
    destinoX=-1;
    destinoY=-1;
    return 0;
  }

  destinoX=x;
  destinoY=y;

  if (x>casX) mover(ABA);
  if (x<casX) mover(ARR);
  if (y>casY) mover(DER);
  if (y<casY) mover(IZQ);
  return 1;
}

/**
 * Controla los movimientos del personaje
 */
int Personaje::controlarMovimiento()
{
  int resultado = Caja::controlarMovimiento();

  moverR(destinoX, destinoY);

  return resultado;
}

/**
 * Detecta si hay una caja en una posicion determinada
 * @param x Posicion horizontal de la caja a detectar
 * @param y Posicion vertical de la caja a detectar
 * @return Devuelve un puntero a la caja que esta en la posicion indicada
 */
Caja* Personaje::detectarCaja(int x, int y)
{
  int i;
  for (i=0;i<nCajas;i++)
    if ((caja[i]->casX==x) && (caja[i]->casY==y)){
      return caja[i];
    }

  return 0;
}
