/*
	Sokonux - 1.0.3
  Copyright (C) 2007  Daniel Ponsoda Montiel

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef Programa_h
#define Programa_h

#include "Sistema.h"
#include "Widget.h"
#include "Texto.h"
#include "Ventana.h"

#define MAX_LINEAS 2000
#define MAX_CARACTERES 1000

/**
 * Clase Programa
 * Esta clase no tiene efecto por si sola. Esta pensada para ser sobreescrita
 * por las clases Menu, Juego, Editor y Records.
 * Sin embargo es la encargada ejecutar el funcionamiento de cada uno de
 * los subprogramas. De cargar y preparar el achivo de texto que
 * se usara en todo el programa, y de crear y mostrar mensajes de informacion.
 */
class Programa {

  public:
    // Constructor y destructor de la clase
    Programa();
    virtual ~Programa();

    // Bucle principal
    virtual int bucle();

    // Prepara el texto del programa
    static void   prepararTexto();

  protected:
    // Comprueba la entrada de usuario
    virtual void  comprobarEntrada();

    // Comprueba el estado del programa
    virtual void  comprobarEstado();

    // Actualiza la pantalla
    virtual void  actualizarPantalla();

    // Sale del programa
    virtual int   terminar();

    // Muestra un mensaje al usuario
    virtual int   mostrarMensaje(Texto* titulo, Texto* mensaje, int flags);

    // Pide informaion al usuario por teclado
    virtual int   pedirInformacion(Texto* titulo, Texto* mensaje,
                                   char **respuesta, int flags);

    // Crea un ventana
    virtual Ventana* crearVentana(Texto* titulo, Texto* mensaje,
                                  int ancho, int alto);


  protected:
    // Flag para salir
    int         m_salir;

    // Pausa empleada para liberar el teclado
    int         m_pausa;

    // Informacion del texto que se usa en el programa
    static Texto**      texto;
    static INFOTEXTO*  infoTexto;
    static int         nTextos;

    // Imagen de fondo
    Widget*     fondo;

};
#endif
