/*
	Sokonux - 1.0.3
  Copyright (C) 2007  Daniel Ponsoda Montiel

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "Records.h"

/**
 * Constructor de la clase records
 */
Records::Records(){
  verAuto=0;
}

/**
 * Destructor de la clase records
 */
Records::~Records(){

}

/**
 * Actualiza la pantalla de los records
 */
void Records::actualizarPantalla()
{
  Texto* tmpTexto;

  // Mostrar el titulo
  TXT_RECORDS->dibujar();

  // Mostrar la lista de records
  TXT_ENTRADAREC->cuadro->y=200;
  DWORD color=0xFF0000;
  for (int i=0;i<10;i++){
    TXT_ENTRADAREC->colorTexto=color;
    color^=0x00FFFF;
    TXT_ENTRADAREC->cuadro->y+=40;

    
    Sistema::textof(&tmpTexto, TXT_ENTRADAREC, i+1,
                    iRecord[i].nombre, iRecord[i].puntos);
    if (tmpTexto){
      tmpTexto->dibujar();
      delete tmpTexto;
      tmpTexto = 0;
    }
  }
}

/**
 * Salir de la pantalla de records
 */
int Records::terminar()
{
  // La pantalla de records se quita sin pedir confirmacion.
  return -1;
}

/**
 * Carga los records desde disco. El nombre del fichero esta definido
 * en la constante FICHERO_TEXTOS en el archivo Records.h
 */
void Records::cargar()
{
  FILE* fichero;
  int nRecords=0;
  // Abrir el fichero de records
  fichero=fopen(FICHERO_RECORDS,"rb");
  if (!fichero) {
    puts("Se crearon records predeterminados.");
    crearPredeterminados();
    return;
  }

  // Leer records
  nRecords = fread(&iRecord,sizeof(IRECORD),10,fichero);
  fclose(fichero);
  if (nRecords!=10){
    puts("Fichero de records no valido.");
    puts("Se crearon records predeterminados.");
    crearPredeterminados();
  }
}

/**
 * Guarda los records en disco.
 */
void Records::guardar()
{
  FILE* fichero;
  // Abrir el fichero de records
  fichero=fopen(FICHERO_RECORDS,"wb");
  if (!fichero){
    puts("Error al escribir el fichero de records");
    return;
  }

  // Leer records
  fwrite(&iRecord,sizeof(IRECORD),10,fichero);
  fclose(fichero);
}

/**
 * Crea nombres y puntos predeterminados para los records cuando no se
 * pudo cargar desde disco.
 */
void Records::crearPredeterminados()
{
  // Crear nombres
  strcpy(iRecord[0].nombre,"Nacho");
  strcpy(iRecord[1].nombre,"Daniel");
  strcpy(iRecord[2].nombre,"Valentin");
  strcpy(iRecord[3].nombre,"Omar");
  strcpy(iRecord[4].nombre,"Toni");
  strcpy(iRecord[5].nombre,"Yovana");
  strcpy(iRecord[6].nombre,"Manolo");
  strcpy(iRecord[7].nombre,"Javi");
  strcpy(iRecord[8].nombre,"Pikeres");
  strcpy(iRecord[9].nombre,"Silvia");

  // Crear puntos
  iRecord[0].puntos=23760;
  iRecord[1].puntos=16544;
  iRecord[2].puntos=15821;
  iRecord[3].puntos=12586;
  iRecord[4].puntos=9211;
  iRecord[5].puntos=8976;
  iRecord[6].puntos=7734;
  iRecord[7].puntos=4734;
  iRecord[8].puntos=2734;
  iRecord[9].puntos=2086;
  
  // Guardar los records recien creados
  guardar();
}

/**
 * Comprueba si una puntuacion esta sobre uno de los records acutales.
 * Si es asi, pide al usario su nombre para introducirlo en la lista
 * @param puntos Puntos que se van a comprobar
 */
void Records::comprobar(int puntos)
{
  int i,j;
  cargar();

  // Buscar en la lista de records
  for (i=0;i<10;i++){
    // Si la puntuacion indicada es mayor que una de la lista
    if (puntos>iRecord[i].puntos){
      // Desplazar las puntuaciones inferiores.
      for (j=9;j>i;j--){
        iRecord[j]=iRecord[j-1];
      }
      iRecord[i].puntos=puntos;
      strcpy(iRecord[i].nombre,"........");
      char* tmp;

      // Pedir el nombre al jugador
      pedirInformacion(TXT_NUEVORECORD, TXT_INTRNOMBRE, &tmp, MSG_ACEPTAR);

      // Guardar el nombre y terminar
      strcpy(iRecord[i].nombre,tmp);
      guardar();
      break;
    }
  }

}

/**
 * Bucle principal de la pantalla de records
 */
int Records::bucle()
{
  cargar();
  return Programa::bucle();
}
