/*
	Sokonux - 1.0.3
  Copyright (C) 2007  Daniel Ponsoda Montiel

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "Sistema.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <SDL/SDL_ttf.h>

void* Sistema::pantalla;
DWORD*  Sistema::teclas=NULL;
void* Sistema::tipo[MAX_TIPOS];
SDL_Surface* Sistema::prueba=NULL;
int Sistema::activo=0;
time_t Sistema::retardoInicio[2000];

/**
 * Prepara las librerias graficas e inicia el modo grafico.
 * predefinido en las macros ANCHO, ALTO, BPP, y FULLSCREEN
 * (Nota: FULLSCREEN 1 = pantalla completa; FULLSCREEN 0 = modo ventana)
 */
Sistema::Sistema(){

  // Inicializar SDL
  if ( SDL_Init( SDL_INIT_VIDEO ) < 0 )
      exit(1);

  // Entrar en modo grafico creando la pantalla visible
  pantalla = (void*)SDL_SetVideoMode(
    ANCHO, ALTO, BPP,SDL_HWSURFACE|SDL_DOUBLEBUF
#ifdef FULLSCREEN
    |SDL_FULLSCREEN
#endif
    );

  if ( !pantalla )
      exit(2);

  // Inicializar manejador de texto SDL_TTF
  if ( TTF_Init() < 0 ) exit(5);

  // Cargar fuentes
  tipo[ARIAL12] = (void*)TTF_OpenFont("ttf/arial.ttf",12);
  if (!tipo[ARIAL12]) exit(6);

  tipo[ARIALB12] = (void*)TTF_OpenFont("ttf/arial.ttf",14);
  if (!tipo[ARIALB12]) exit(6);

  tipo[OPCIONES] = (void*)TTF_OpenFont("ttf/typeo.ttf",32);
  if (!tipo[OPCIONES]) exit(7);

  tipo[TITULO] = (void*)TTF_OpenFont("ttf/quixotic.ttf",120);
  if (!tipo[TITULO]) exit(8);

  tipo[RECORDS] = (void*)TTF_OpenFont("ttf/typeo.ttf",64);
  if (!tipo[RECORDS]) exit(9);

}

/**
  * Libera la memoria utilizada por los graficos y descarga las librerias.
  */
Sistema::~Sistema(){
  if (pantalla) SDL_FreeSurface((SDL_Surface*)pantalla);
 // if (pantallaOculta) SDL_FreeSurface((SDL_Surface*)pantallaOculta);
  if (TTF_WasInit()) TTF_Quit();
  if (SDL_WasInit(SDL_INIT_EVERYTHING)) SDL_Quit();
}

/**
 * Carga una imagen en formato BMP desde disco.
 * @param nombreFichero Nombre del fichero BMP que contiene la imagen.
 * @param colorClave Color que sera interpretado como transparente.
 * @return Devuelve un puntero a la superficie grafica.
 * El puntero sera de tipo void*, para independizarlo del resto del programa.
 * @see Sistema::liberarImagen
 * @see Sistema::dibujarImagen
 * @see Sistema::borrarImagen
 */
void* Sistema::cargarImagen(char* nombreFichero, int* ancho, int* alto,
                            DWORD colorClave)
{
  int r,g,b;
  SDL_Surface* imagen;

  imagen = SDL_LoadBMP(nombreFichero);
  if (imagen == NULL){
    puts("Error al cargar una imagen");
    exit(4);
  }

  *ancho=imagen->w;
  *alto=imagen->h;

  r = (colorClave & 0xff0000)>>16;
  g = (colorClave & 0x00ff00)>>8;
  b = colorClave & 0x0000ff;

  SDL_SetColorKey(imagen, SDL_SRCCOLORKEY|SDL_RLEACCEL,
      SDL_MapRGB(imagen->format, r,g,b));

  return (void*)imagen;
}

/**
 * Libera una imagen previamente cargada en memoria.
 * Se emplea fundamentalmente en los destructores de las clases.
 * @param superficie Puntero a la imagen que se va a liberar.
 * @see Sistema::cargarImagen
 * @see Sistema::dibujarImagen
 * @see Sistema::borrarImagen
 */
void Sistema::liberarImagen(void *superficie){
  if (superficie){
    SDL_FreeSurface((SDL_Surface*)superficie);
  }
}

/**
 * Dibuja una imagen sobre la pantalla oculta.
 * Para mostrarla en pantalla se debera llamar al metodo volcarPantalla.
 * @param superficie Puntero a la imagen que se dibujara.
 * @param x Posicion horizontal de la pantalla.
 * @param y Posicion vertical de la pantalla.
 * @see Sistema::volcarPantalla
 */
void Sistema::dibujarImagen(void* superficie, int x, int y)
{
  SDL_Rect posicion;
  posicion.x = x;
  posicion.y = y;
  SDL_BlitSurface((SDL_Surface*)superficie, 0,
                  (SDL_Surface*)pantalla, &posicion);
}

/**
 * Borra la superficie oculta de la pantalla.
 * @param color Color con el que se borrara la superficie.
 * @see Sistema::borrarImagen
 * @see Sistema::volcarPantalla
 */
void Sistema::borrarPantallaOculta(DWORD color)
{
  borrarImagen(pantalla,color);
}

/**
 * Borra el contenido de una imagen indicada.
 * @param superficie Puntero a la imagen que se quiere borrar.
 * @param color Color con el que se borrara.
 * @see Sistema::dibujarImagen
 * @see Sistema::borrarImagen
 */
void Sistema::borrarImagen(void* superficie, DWORD color)
{
  int r,g,b;
  r = (color & 0xff0000)>>16;
  g = (color & 0x00ff00)>>8;
  b = color & 0x0000ff;

  SDL_FillRect((SDL_Surface*)superficie,0,
      SDL_MapRGB(((SDL_Surface*)superficie)->format,r,g,b));

}

/**
 * Copia el contenido de una imagen sobre otra.
 * Las dimensiones de la imagen destino no se alteraran aunque el origen
 * sea de tamao distinto. Solo se copian los bits de una a otra.
 */
void Sistema::copiarImagen(void* origen, void* destino)
{
  SDL_BlitSurface((SDL_Surface*)origen,0,(SDL_Surface*)destino,0);
}

/**
 * Crea una nueva superficie de la resolucion indicada.
 * @param ancho Anchura de la superficie
 * @param alto Altura de la superficie
 * @param bpp Profundidad de color de la superficie
 * @return Puntero a la superficie grafica creada
 */
void* Sistema::crearSuperficie(int ancho, int alto, int bpp)
{
  return SDL_CreateRGBSurface(SDL_SWSURFACE,
     ancho,alto,bpp, RMASK,GMASK,BMASK,AMASK);
}

/**
 * Copia la pantalla oculta sobre la pantalla visible y la muestra.
 * @see Sistema::borrarPantallaOculta
 */
void Sistema::volcarPantalla()
{
  /*SDL_BlitSurface((SDL_Surface*)pantallaOculta, 0,
    (SDL_Surface*)pantallaVisible, 0);*/
  SDL_Flip( (SDL_Surface*)pantalla );

  borrarPantallaOculta(0);
}

DWORD Sistema::comprobarBotonesRaton() {
    int x,y;
    SDL_PumpEvents();

    return(SDL_GetMouseState(&x,&y));
}

/**
 * Comprueba la posicion y los botones pulsados del raton.
 * @param *x Puntero a la posicion horizontal
 * @param *y Puntero a la posicion vertical
 * @return Indica con un formato especial, cuales de los
 * botones del raton han sido pulsados.
 * @see Sistema::detectarRaton
 */
DWORD Sistema::comprobarRaton(int *x, int *y) {
    SDL_PumpEvents();

    return(SDL_GetMouseState(x,y)/*&SDL_BUTTON(1)*/);
}

DWORD Sistema::ratonClickArriba() {
  SDL_Event evento;

  while(SDL_PollEvent(&evento))
  {
    if (evento.type==SDL_MOUSEBUTTONUP)
    {
      return 1;
    }
  }
  return 0;
}

/**
 * Detecta si el raton se encuentra en un area determinada de la pantalla.
 * @param x Posicion horizontal del area
 * @param y Posicion vertical del area
 * @param ancho Anchura del area
 * @param alto Altura del area
 */
DWORD Sistema::detectarRaton(CUADRO cuadro)
{
  int x=cuadro.x      , y=cuadro.y;
  int ancho=cuadro.an , alto=cuadro.al;
  int ratonx, ratony;
  int boton=comprobarRaton(&ratonx, &ratony);

  if(detectarColision(x,y,ancho,alto,ratonx,ratony,1,1))
  {
    if (boton) {
      return MSG_RATON_CLICK;
    }
    else return MSG_DETECTADO_RATON;
  }

  return 0;
}

/**
 * Detecta si el raton se encuentra en un area determinada de la pantalla.
 * De forma estricta, es decir comprobando la silueta de una imagen.
 * @param x Posicion horizontal del area
 * @param y Posicion vertical del area
 * @param imagen Imagen a comprobar
 */
DWORD Sistema::detectarRaton(int imgX, int imgY, void* imagen)
{
  int x,y, rX,rY;
  int boton;

  // Obtener los datos de la imagen
  //COLOR32* pixels = (COLOR32*)((SDL_Surface*)imagen)->pixels;
  COLOR24* pixels = (COLOR24*)((SDL_Surface*)imagen)->pixels;
  int ancho       = ((SDL_Surface*)imagen)->w;
  int alto        = ((SDL_Surface*)imagen)->h;

  // Si no esta en el cuadro, no nos molestamos en comprobar la silueta
  CUADRO cuadro={imgX,imgY,ancho,alto};
  if (!(detectarRaton(cuadro)))
    return 0;


  // Obtener la posicion del raton
  boton = comprobarRaton(&rX,&rY);

  // Comprobar la silueta de la imagen
  for (x=0;x<ancho;x++){
    for (y=0;y<alto;y++){
      //if (pixels[y*ancho+x]){
      if (pixels[y*ancho+x].r || pixels[y*ancho+x].g || pixels[y*ancho+x].b){

        if ( (rX == x+imgX) && (rY == y+imgY) ){
          if (boton) {
            return MSG_RATON_CLICK;
          }
          else return MSG_DETECTADO_RATON;
        }
      }
    }
  }

  return 0;
}

/**
 * Comprueba si hay una tecla pulsada
 * @param c Tecla a comprobar
 * @return 1 si esta pulsada; 0 si no esta pulsada
 */
bool Sistema::comprobarTeclado(int c)
{
  SDL_PumpEvents();
  Uint8 *keys = SDL_GetKeyState(NULL);
  if (keys[c] == 1)
    return 1;
  else
    return 0;
}

DWORD Sistema::comprobarTeclado()
{
  SDL_Event evento;

  while(SDL_PollEvent(&evento))
  {
    if (evento.type==SDL_KEYDOWN)
    {
      if (evento.key.keysym.sym)
        return (DWORD)evento.key.keysym.sym;
    }
  }
  return 0;
}

bool Sistema::comprobarTeclado(int c,int retardo) {
  SDL_PumpEvents();
  Uint8 *keys = SDL_GetKeyState(NULL);
  if (keys[c] == 1){
    if (!retardoInicio[c]){
      retardoInicio[c]=clock();
    }
    else{
      if ((clock()-retardoInicio[c])<retardo) return 0;
      else {
        retardoInicio[c]=0;
        return 0;
      }
    }
    return 1;
  }

  else{
    retardoInicio[c]=0;
    return 0;
  }
}

/**
 * Limpia la memoria del teclado
 * @see Sistem::comprobarTeclado
 */
void Sistema::vaciarBufferTeclado(){
  DWORD letra;
  do{
    letra=Sistema::comprobarTeclado();
  }while(letra!=0);
}

/**
 * Escribe texto en pantalla
 * @param *frase Cadena que contiene el texto
 * @param x Posicion horizontal de la pantalla
 * @param y Posicion vertical de la pantalla
 * @param color Color con que se dibujara
 * @param *fuente Tipo de letra que se usara
 * @see Sistema::obtenerTexto
 */
void Sistema::escribirTexto(char *frase, int x,int y,int color, DWORD fuente)
{
  SDL_Surface* resultado=NULL;

  // Obtener el color frontal
  SDL_Color fgcolor;
  fgcolor.r = (color & 0xff0000)>>16;
  fgcolor.g = (color & 0x00ff00)>>8;
  fgcolor.b = color & 0x0000ff;

  // Crear la imagen de texto
  resultado = TTF_RenderText_Blended((TTF_Font*)tipo[fuente],frase,fgcolor);
  dibujarImagen(resultado,x,y);
  liberarImagen((void*)resultado);
}

void Sistema::obtenerTamTexto(char *frase, DWORD fuente,
               int *imgAncho, int *imgAlto)
{
  SDL_Surface *resultado=NULL;
  SDL_Color fgcolor;

  // Crear la imagen de texto
  resultado = TTF_RenderText_Blended((TTF_Font*)tipo[fuente],frase,fgcolor);

  // Dar los valores de retorno
  *imgAncho = resultado->w;
  *imgAlto  = resultado->h;

  liberarImagen((void*)resultado);
}

/**
 * Comprueba si dos areas estan solapadas
 * Los cuatro primeros parametros son respectivamente las posiciones
 * horizontal y vertical, y el ancho y el alto del area 1.
 * Los otros cuatro parametros son los mismos, correspondientes al area 2.
 * @return Devuelve 1 si estan en colision y 0 si no lo estan
 */
DWORD Sistema::detectarColision(int x1, int y1, int an1, int al1,
                              int x2, int y2, int an2, int al2) {
  if ((x2+an2 > x1) && (y2+al2 > y1) && (x1+an1 > x2) && (y1+al1 > y2))
    return 1;
  else
    return 0;
}


/**
 * Realiza una pausa
 * @param milisegundos Tiempo en milisegundos que durara la pausa
 */
void Sistema::pausa(int milisegundos){
  SDL_Delay(milisegundos);
}

/**
 * Dibuja un cuadro en la pantalla oculta
 * @param x Posicion horizontal
 * @param y Posicion vertical
 * @param ancho Anchura del cuadro
 * @param alto Altura del cuadro
 * @param colorFondo Color con el que se rellenara el area del cuadro
 * @param colorBorde Color del borde
 */
void Sistema::dibujarCuadro(CUADRO cuadro,
              DWORD colorFondo, DWORD colorBorde)
{
  int i,j;
  int x=cuadro.x      , y=cuadro.y;
  int ancho=cuadro.an , alto=cuadro.al;

  // Si los colores son transparentes, volver.
  if ((colorFondo==COLOR_CLAVE) && (colorBorde==COLOR_CLAVE)) return;

  // Evitar que se cree una superficie vacia
  if ((!ancho) || (!alto)) return;
  if (ancho<2) ancho=2;
  if (alto<2) alto=2;

  // Crear una superfice para contener el cuadro
  SDL_Surface *bufferCuadro=SDL_CreateRGBSurface(SDL_HWSURFACE,
    ancho,alto,BPP, 0x00FF0000,0x0000FF00,0x000000FF,0);
  if ( !bufferCuadro) return;

  // Borrar el cuadro con el color de fondo
  if (colorFondo!=COLOR_CLAVE)
    borrarImagen(bufferCuadro,colorFondo);

  // Dibujar el borde con el color de borde
  if (colorBorde!=COLOR_CLAVE){
    Uint32* pixels=(Uint32*)bufferCuadro->pixels;
    for (i=0;i<ancho;i++) {
      pixels[i]=colorBorde;                 // Linea superior
      pixels[(alto-1)*ancho+i]=colorBorde;  // Linea inferior
    }
    for (j=1;j<alto-1;j++){
      pixels[j*ancho]=colorBorde;           // Linea izquierda
      pixels[j*ancho+ancho-1]=colorBorde;   // Linea derecha
    }
  }

  // Dibujar el cuadro en la pantalla oculta
  dibujarImagen(bufferCuadro,x,y);
  liberarImagen((void*)bufferCuadro);

}


int Sistema::cargarTexto(char *nombreFichero, char *marcador,
                                INFOTEXTO** textoLeido)
{
  FILE *fichero;
  char* cadena = new char[10001];
  int i=0;
  bool leyendo=0;
  INFOTEXTO* infotexto=NULL;
  int nTextos;

  // Abrir el fichero de textos
  fichero=fopen(nombreFichero,"rt");
  if (!fichero) {
   puts("Error al cargar el achivo de textos");
   exit(1);
  }

  // Contar numero de lineas del fichero
  nTextos=0;
  while(!feof(fichero)){
    fgets(cadena,10000,fichero);
    if(!feof(fichero)) {
      if (!leyendo){
        if (strstr(cadena,marcador)) leyendo=1;
      }
      else{
        if (strstr(cadena,"[FIN]")) break;
        nTextos++;
      }
    }
  }
  leyendo=0;

  // Reservar espacio para el texto
  infotexto=new INFOTEXTO[nTextos];
  if (!infotexto){
    puts("Error al crear espacio para el listado de textos");
    exit(2);
  }

  // Leer texto;
  fseek(fichero,0,SEEK_SET);
  while(!feof(fichero)){
    fgets(cadena,1000,fichero);
    if (!feof(fichero)){
      if (!leyendo){
        if (strstr(cadena,marcador)) leyendo=1;
      }
      else{
        if (strstr(cadena,"[FIN]")) break;
        sscanf(cadena,"%d.%x.%x.%x.%d.%d.%d.%d.%d:", &infotexto[i].id,
          &infotexto[i].colorTexto, &infotexto[i].colorSombra,
          &infotexto[i].colorRaton, &infotexto[i].fuente,
          &infotexto[i].despSombra, &infotexto[i].flags,
          &infotexto[i].x, &infotexto[i].y);

        cadena=strstr(cadena," ")+1;

        infotexto[i].cadena = new char[strlen(cadena)+1];
        strcpy(infotexto[i].cadena, cadena);
        infotexto[i].cadena[strlen(infotexto[i].cadena)-1]=0;

        i++;
      }
    }
  }
  *textoLeido=infotexto;
  return nTextos;
}

int Sistema::textof(Texto** resultado, Texto* formato,...)
{
  INFOTEXTO tmpInfoTexto={0,
                          new char[1000],
                          formato->cuadro->x, formato->cuadro->y,
                          formato->colorTexto,
                          formato->colorSombra,
                          formato->colorRaton,
                          formato->fuente,
                          formato->despSombra,
                          formato->flags};

  va_list varList;

  va_start(varList, formato);

  vsprintf(tmpInfoTexto.cadena, formato->cadena, varList);

  *resultado=new Texto(0,tmpInfoTexto);
  return strlen(tmpInfoTexto.cadena);
}
