/*
	Sokonux - 1.0.3
  Copyright (C) 2007  Daniel Ponsoda Montiel

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef Sistema_h
#define Sistema_h

#include "defines.h"
#include "Texto.h"
#include <SDL/SDL.h>
#include <time.h>

#define ANCHO   1024
#define ALTO     768
#define BPP       32

#ifndef DEBUG
  #define FULLSCREEN
#endif

#define tiempo_t          time_t
#define PASOS_POR_SEGUNDO CLOCKS_PER_SEC

#define AMASK 0xFF000000
#define RMASK 0x00FF0000
#define GMASK 0x0000FF00
#define BMASK 0x000000FF
#define COLOR_CLAVE 0

#define MSG_DETECTADO_RATON  1
#define MSG_RATON_CLICK      2

#define TECLA_1   SDLK_1
#define TECLA_2   SDLK_2
#define TECLA_3   SDLK_3
#define TECLA_4   SDLK_4
#define TECLA_5   SDLK_5
#define TECLA_ESC SDLK_ESCAPE
#define TECLA_RET SDLK_RETURN
#define TECLA_DEL SDLK_BACKSPACE
#define TECLA_IZQ SDLK_LEFT
#define TECLA_DER SDLK_RIGHT
#define TECLA_ARR SDLK_UP
#define TECLA_ABA SDLK_DOWN


#define ARIAL12   0
#define ARIALB12  1
#define OPCIONES  2
#define TITULO    3
#define RECORDS   4
#define MAX_TIPOS 5


/**
 * Esta clase reune una serie de metodos estaticos que proporcionan
 * servicios de hardware de forma transparente a toda la aplicacin
 */
class Sistema {
  public:
    // Constructor de la clase
    Sistema();

    // Destructor de la clase
    virtual ~Sistema();

    //
    // Metodos para gestion de graficos
    //
    static void* cargarImagen        (char* nombreFichero, int* ancho,int* alto,
                                      DWORD colorClave=COLOR_CLAVE);
    static void  dibujarImagen       (void* superficie, int x, int y);
    static void  volcarPantalla      ();
    static void  copiarImagen        (void* origen, void* destino);
    static void* crearSuperficie     (int ancho, int alto, int bpp);
    static void  borrarPantallaOculta(DWORD color);
    static void  borrarImagen        (void* superficie, DWORD color);
    static void  liberarImagen       (void* superficie);
    static DWORD detectarColision    (int x1, int y1, int an1, int al1,
                                      int x2, int y2, int an2, int al2);

    static void  dibujarCuadro       (CUADRO cuadro,
                                      DWORD colorFondo, DWORD colorBorde);


    //
    // Metodos para gestion de la entrada de teclado y raton
    //
    static bool   comprobarTeclado      (int c);
    static bool   comprobarTeclado      (int c,int retardo);
    static DWORD  comprobarTeclado      ();
    static void   vaciarBufferTeclado   ();
    static DWORD  comprobarRaton        (int *x, int *y);
    static DWORD  comprobarBotonesRaton ();
    static DWORD  detectarRaton         (CUADRO cuadro);
    static DWORD  detectarRaton         (int x, int y, void* imagen);
    static DWORD  ratonClickArriba      ();
    //
    // Metodos para el manejo de texto
    //
    static void   escribirTexto         (char *frase, int x, int y,
                                         int color, DWORD fuente);

    static int    cargarTexto           (char *nombreFichero, char* marcador,
                                         INFOTEXTO** textoLeido);

    static void   obtenerTamTexto       (char *frase, DWORD fuente,
                                         int *imgAncho, int *imgAlto);

    static int    textof                (Texto** resultado,
                                          Texto* formato,...);

    //
    // Control de tiempo
    //
    static void     pausa               (int milisegundos);
    static tiempo_t reloj               (){ return clock(); };


  public:
    // Mapa del teclado
    static DWORD*       teclas;

    // Buffers de pantalla
    static void*        pantalla;

    // Tipos de letra
    static void*        tipo[MAX_TIPOS];

    static SDL_Surface* prueba;
    static int          activo;

    static time_t       retardoInicio[2000];

};
#endif
