/*
	Sokonux - 1.0.3
  Copyright (C) 2007  Daniel Ponsoda Montiel

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "Ventana.h"
#include "Sistema.h"

#include <string.h>

/**
 * Constructor de la clase ventana
 * @param cuadroPadre Indica cual es su cuadro padre (en general a que ventana
 *                    pertenece). Si esta a 0, el padre es la pantalla
 * @param titulo      Texto que aparece en la barra de titulo
 * @param x           Posicion horizontal relativa al padre
 * @param y           Posicion vertical relativa al padre
 * @param an          Anchura de la ventana
 * @param al          Altura de la ventana
 * @param flags       Opciones de dibujado para la ventana (ver clase Widget)
 */
Ventana::Ventana (CUADRO** cuadroPadre, Texto* titulo,
                  int x, int y, int an, int al, int flags)
       : Widget(cuadroPadre, x, y, an, al)
{
  // Establecer valores por defecto a los atributos
  this->flags       = flags;
  colorBarraTitulo  = DF_COLORBARRATITULO;
  colorTextoTitulo  = DF_COLORTEXTOTITULO;
  colorFondo        = DF_COLORFONDO;
  colorBorde        = DF_COLORBORDE;
  colorRaton        = DF_COLORRATON;

  arrastrando   = 0;
  nBotones      = 0;
  nTextos       = 0;
  nCuadrosTexto = 0;

  centrar(flags);

  // Preparar el titulo de la ventana
  if (titulo->cadena){

    barraTitulo = new Boton(&cuadro, titulo, 1, 1, cuadro->an - 2, 20);
    barraTitulo->colorTexto = colorTextoTitulo;
    barraTitulo->colorFondo = colorBarraTitulo;
    barraTitulo->colorBorde = colorBarraTitulo;
    barraTitulo->flags = WID_DETECTARRATON;
    barraTitulo->texto->flags ^=WID_DETECTARRATON;
    barraTitulo->texto->fuente = ARIALB12;

  }
  else barraTitulo=NULL;
}

/**
 * Destructor de la clase ventana
 */
Ventana::~Ventana()
{
  int i;
  if (barraTitulo)              delete barraTitulo;
  for (i=0;i<nTextos;i++)       delete texto[i];
  for (i=0;i<nBotones;i++)      delete boton[i];
  for (i=0;i<nCuadrosTexto;i++) delete cuadroTexto[i];
}

/**
 * Dibuja la ventana en la posicion actual
 */
void Ventana::dibujar(){

  int todo;

  comprobarEstado();

  // Dibujar el cuadro de la ventana
  Widget::dibujar();

  // Dibujar la barra de titulo
  if (barraTitulo)                            barraTitulo->dibujar();

  // Escribir todos los textos
  for (todo=0; todo < nTextos; todo++)        texto[todo]->dibujar();

  // Dibujar todos los botones
  for (todo=0; todo < nBotones; todo++)       boton[todo]->dibujar();

  // Dibujar todos los cuadros de texto
  for (todo=0; todo < nCuadrosTexto; todo++)  cuadroTexto[todo]->dibujar();

}

/**
 * Comprueba el estado de la ventana
 */
int Ventana::comprobarEstado()
{
  int ratonx,ratony;
  int detectar;
  int mensaje=0;

  if (flags&WID_ARRASTRABLE) {
    detectar=barraTitulo->detectarRaton();

    // Si se esta arrastrando
    if ((detectar==MSG_RATON_CLICK) || (arrastrando)){
      if (!arrastrando){
        Sistema::comprobarRaton(&posIniX,&posIniY);
        arrastrando=1;
      }
      else{
        // Comprobar si se sigue arrastrando
        arrastrando=Sistema::comprobarRaton(&ratonx,&ratony);

        // Actualizar la posicion de la ventana
        cuadro->x+=ratonx-posIniX;
        cuadro->y+=ratony-posIniY;

        // Reestablecer el punto de referencia
        posIniX=ratonx;
        posIniY=ratony;
      }
    }
  }

  return mensaje;
}

/**
 * Agrega texto a la ventana
 * @param cadena Texto que se va a agregar
 * @param x Posicion horizontal relativa a la ventana
 * @param y Posicion vertical relativa a la ventana
 * @param color Color del texto
 * @param fuente Tipo de letra del texto
 * @return puntero al texto creado
 */
Texto* Ventana::agregarTexto(Texto* cadena, int x,int y, DWORD color, int fuente)
{
  y+=20;
  INFOTEXTO infoTexto;
  infoTexto.cadena = new char[strlen(cadena->cadena)+1];
  strcpy(infoTexto.cadena,cadena->cadena);
  infoTexto.x=x;
  infoTexto.y=y;
  infoTexto.colorTexto=color;
  infoTexto.fuente=fuente;
  infoTexto.flags=0;

  texto[nTextos] = new Texto(&cuadro,infoTexto);

  nTextos++;

  return texto[nTextos-1];
}

/**
 * Agrega un boton a la ventana a partir de un fichero de imagen
 * @param nombreFichero Nombre del fichero de imagen
 * @param x Posicion horizontal relativa a la ventana
 * @param y Posicion vertical relativa a la ventana
 * @return puntero al boton creado
 */
Boton* Ventana::agregarBoton(char* nombreFichero, int x, int y )
{
  y+=20;
  boton[nBotones] = new Boton(&cuadro, nombreFichero, x, y);

  nBotones++;

  return boton[nBotones-1];
}

/**
 * Agrega un boton a la ventana indicando el tamao y el texto
 * @param texto Texto que contendra el boton
 * @param x Posicion horizontal relativa a la ventana
 * @param y Posicion vertical relativa a la ventana
 * @param ancho Anchura del boton
 * @param alto Altura del boton
 * @return puntero al boton creado
 */
Boton* Ventana::agregarBoton(Texto* texto, int x, int y, int ancho, int alto)
{
  y+=20;
  boton[nBotones] = new Boton(&cuadro, texto, x, y, ancho, alto);

  nBotones++;

  return boton[nBotones-1];
}

/**
 * Agrega un cuadro de texto a la ventana indicando el tamao
 * @param x Posicion horizontal relativa a la ventana
 * @param y Posicion vertical relativa a la ventana
 * @param ancho Anchura del cuadro de texto
 * @param alto Altura del cuadro de texto
 * @param maxLetras Maximo numero de caracteres que caberan en el cuadro.
 * @return puntero al cuadro de texto creado
 */
CuadroTexto* Ventana::agregarCuadroTexto(int x, int y, int ancho,
                                         int alto, int maxLetras)
{
  y+=20;
  cuadroTexto[nCuadrosTexto] =
      new CuadroTexto(&cuadro, x, y, ancho, alto, maxLetras);

  nCuadrosTexto++;

  return cuadroTexto[nCuadrosTexto-1];
}

